/**
 * edg-gridftp-base-rename
 * 
 * A command line interface to the GridFTP libraries to allow a user
 * to move a file/directory on a remote server.  End users should invoke 
 * the perl wrapper rather than this. 
 *
 * This command returns 0 if the operations was a success, 1 otherwise.
 * The underlying error message from the GridFTP library should be printed 
 * on stderr for unsuccessful operations.
 *
 * Charles Loomis (2002-10-18)
 * CERN IT-DM/SMD (2008-02-25)
 *
 */

/* Include Files */
#include <stdlib.h>
#include <unistd.h>

#include "globus_ftp_client.h"
#include "edg-gridftp-base-common.h"

const char *prog;

/* Main function. */
int main(int argc, char **argv) {

  char *usage = "Usage: edg-gridftp-base-rename [-t <timeout in seconds>] <source gsiftp URL> <dest. URL>\n";

  char *url_source;
  char *url_dest;
  globus_url_t url_struct;

  int c;
  int status;
  int timeout;
  globus_result_t gresult;

  globus_ftp_client_handle_t ftp_handle;
  globus_ftp_client_handleattr_t ftp_handle_attr;
  globus_ftp_client_operationattr_t ftp_op_attr;

  monitor_t monitor;

  prog = argv[0];

  /* Check the argument. Single ftp, gsiftp, or gridftp URL. */
  timeout = 120;

  opterr = 0;
  optind = 1;
  while((c = getopt(argc, argv, "t:")) != -1) {
    switch(c) {
      case 't':
          timeout = atoi(optarg);
          if (timeout <= 0) {
            my_fprintf(stderr, "%s: invalid timeout %s\n", prog, optarg);
            return FAIL;
          }                                                         
          break;
      default:
          my_fprintf(stderr,usage);
          return FAIL;
          break;
    }
  }
  argc -= optind;
  argv += optind;

  if (argc != 2) {
    my_fprintf(stderr,usage);
    return FAIL;
  }
  url_source = argv[0];
  url_dest = argv[1];

  setup_timer_and_signals(timeout);

  /* Try parsing the argument to ensure that it is OK. */
  status = globus_url_parse(url_dest, &url_struct);
  if (status != GLOBUS_SUCCESS) {
    my_fprintf(stderr, "%s: error parsing destination URL\n",prog);
    return FAIL;
  }
  status = globus_url_parse(url_source, &url_struct);
  if (status != GLOBUS_SUCCESS) {
    my_fprintf(stderr, "%: error parsing source URL\n",prog);
    return FAIL;
  }

  /* Initialize the lock. */
  globus_mutex_init(&monitor.mutex, GLOBUS_NULL);
  globus_cond_init(&monitor.cond, GLOBUS_NULL);
  monitor.error = GLOBUS_FALSE;
  monitor.done = GLOBUS_FALSE;
  monitor.monitor = NULL;

  /* Activate the globus ftp client module. */
  status = globus_module_activate(GLOBUS_FTP_CLIENT_MODULE);
  if (status != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not activate ftp client module\n",
			prog);
    return FAIL;
  }


  /* Create an empty ftp client attribute. */
  gresult = globus_ftp_client_handleattr_init(&ftp_handle_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not create ftp handle attributes\n",
			prog);
    return FAIL;
  }


  /* Create an ftp client handle. */
  gresult = globus_ftp_client_handle_init(&ftp_handle,&ftp_handle_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not create ftp handle\n",
			prog);
    return FAIL;
  }


  /* Create an ftp operations attribute. */
  gresult = globus_ftp_client_operationattr_init(&ftp_op_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
  			"%s: could not create operations attribute\n",
  			prog);
    return FAIL;
  }
  
  /* If URL contains username and password, then set the authorization 
     to use this rather than the default GSI authorization. */
  if (url_struct.user || url_struct.password) {
    gresult = 
      globus_ftp_client_operationattr_set_authorization(&ftp_op_attr,
							GSS_C_NO_CREDENTIAL,
							url_struct.user,
							url_struct.password,
							"",
							"");
    if (gresult != GLOBUS_SUCCESS) {
      my_fprintf(stderr, 
			  "%s: could not set authorization info\n",
			  prog);
      return FAIL;
    }
  }

  /* Now do the real work, make the directory. */
  gresult = globus_ftp_client_move(&ftp_handle,
				   url_source,
				   url_dest,
				   &ftp_op_attr,
				   &callback,
				   (void *) &monitor);

  /* Wait for the result. */
  globus_mutex_lock(&monitor.mutex);
  while (!monitor.done) {
    globus_cond_wait(&monitor.cond, &monitor.mutex);
  }
  globus_mutex_unlock(&monitor.mutex);


  /* Clean up.  Silently ignore errors since we already have the result
     of the ftp operation itself. */
  gresult = globus_ftp_client_operationattr_destroy(&ftp_op_attr);
  gresult = globus_ftp_client_handle_destroy(&ftp_handle);
  gresult = globus_ftp_client_handleattr_destroy(&ftp_handle_attr);
  status = globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

  return monitor.error ? FAIL : OK;

} /* main() */
